@@ -1,10 +1,14 @@
-<?php /** @var $l OC_L10N */ ?>
+<?php /** @var $l \OCP\IL10N */ ?>
 <div id="controls">
 	<div id="file_action_panel"></div>
 </div>
 <div id='notification'></div>
 
-<div id="emptycontent" class="hidden"><?php p($l->t( 'You don\'t have any external storages' )); ?></div>
+<div id="emptycontent" class="hidden">
+	<div class="icon-external"></div>
+	<h2><?php p($l->t('No external storage configured')); ?></h2>
+	<p><?php p($l->t('You can add external storages in the personal settings')); ?></p>
+</div>
 
 <input type="hidden" name="dir" value="" id="dir">
 
