@@ -13,6 +13,7 @@
  * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
  * @author Scrutinizer Auto-Fixer <auto-fixer@scrutinizer-ci.com>
  * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Victor Dubiniuk <dubiniuk@owncloud.com>
  * @author Vincent Petry <pvince81@owncloud.com>
  *
  * @copyright Copyright (c) 2015, ownCloud, Inc.
@@ -40,7 +41,9 @@
 
 namespace OCA\Files_Versions;
 
+use OCA\Files_Versions\AppInfo\Application;
 use OCA\Files_Versions\Command\Expire;
+use OCP\Lock\ILockingProvider;
 
 class Storage {
 
@@ -67,6 +70,9 @@ class Storage {
 		//until the end one version per week
 		6 => array('intervalEndsAfter' => -1,      'step' => 604800),
 	);
+	
+	/** @var \OCA\Files_Versions\AppInfo\Application */
+	private static $application;
 
 	public static function getUidAndFilename($filename) {
 		$uid = \OC\Files\Filesystem::getOwner($filename);
@@ -323,6 +329,9 @@ class Storage {
 			if (self::copyFileContents($users_view, 'files_versions' . $filename . '.v' . $revision, 'files' . $filename)) {
 				$files_view->touch($file, $revision);
 				Storage::scheduleExpire($uid, $file);
+				\OC_Hook::emit('\OCP\Versions', 'rollback', array(
+					'path' => $filename,
+				));
 				return true;
 			} else if ($versionCreated) {
 				self::deleteVersion($users_view, $version);
@@ -342,10 +351,31 @@ class Storage {
 	 * @return bool true for success, false otherwise
 	 */
 	private static function copyFileContents($view, $path1, $path2) {
+		/** @var \OC\Files\Storage\Storage $storage1 */
 		list($storage1, $internalPath1) = $view->resolvePath($path1);
+		/** @var \OC\Files\Storage\Storage $storage2 */
 		list($storage2, $internalPath2) = $view->resolvePath($path2);
 
-		$result = $storage2->moveFromStorage($storage1, $internalPath1, $internalPath2);
+		$view->lockFile($path1, ILockingProvider::LOCK_EXCLUSIVE);
+		$view->lockFile($path2, ILockingProvider::LOCK_EXCLUSIVE);
+
+		// TODO add a proper way of overwriting a file while maintaining file ids
+		if ($storage1->instanceOfStorage('\OC\Files\ObjectStore\ObjectStoreStorage') || $storage2->instanceOfStorage('\OC\Files\ObjectStore\ObjectStoreStorage')) {
+			$source = $storage1->fopen($internalPath1, 'r');
+			$target = $storage2->fopen($internalPath2, 'w');
+			list(, $result) = \OC_Helper::streamCopy($source, $target);
+			fclose($source);
+			fclose($target);
+
+			if ($result !== false) {
+				$storage1->unlink($internalPath1);
+			}
+		} else {
+			$result = $storage2->moveFromStorage($storage1, $internalPath1, $internalPath2);
+		}
+
+		$view->unlockFile($path1, ILockingProvider::LOCK_EXCLUSIVE);
+		$view->unlockFile($path2, ILockingProvider::LOCK_EXCLUSIVE);
 
 		return ($result !== false);
 	}
@@ -411,6 +441,38 @@ class Storage {
 		return $versions;
 	}
 
+	/**
+	 * Expire versions that older than max version retention time
+	 * @param string $uid
+	 */
+	public static function expireOlderThanMaxForUser($uid){
+		$expiration = self::getExpiration();
+		$threshold = $expiration->getMaxAgeAsTimestamp();
+		$versions = self::getAllVersions($uid);
+		if (!$threshold || !array_key_exists('all', $versions)) {
+			return;
+		}
+
+		$toDelete = [];
+		foreach (array_reverse($versions['all']) as $key => $version) {
+			if (intval($version['version'])<$threshold) {
+				$toDelete[$key] = $version;
+			} else {
+				//Versions are sorted by time - nothing mo to iterate.
+				break;
+			}
+		}
+
+		$view = new \OC\Files\View('/' . $uid . '/files_versions');
+		if (!empty($toDelete)) {
+			foreach ($toDelete as $version) {
+				\OC_Hook::emit('\OCP\Versions', 'preDelete', array('path' => $version['path'].'.v'.$version['version']));
+				self::deleteVersion($view, $version['path'] . '.v' . $version['version']);
+				\OC_Hook::emit('\OCP\Versions', 'delete', array('path' => $version['path'].'.v'.$version['version']));
+			}
+		}
+	}
+
 	/**
 	 * translate a timestamp into a string like "5 days ago"
 	 * @param int $timestamp
@@ -453,13 +515,15 @@ class Storage {
 			$files = $view->getDirectoryContent($dir);
 
 			foreach ($files as $file) {
+				$fileData = $file->getData();
+				$filePath = $dir . '/' . $fileData['name'];
 				if ($file['type'] === 'dir') {
-					array_push($dirs, $file['path']);
+					array_push($dirs, $filePath);
 				} else {
-					$versionsBegin = strrpos($file['path'], '.v');
+					$versionsBegin = strrpos($filePath, '.v');
 					$relPathStart = strlen(self::VERSIONS_ROOT);
-					$version = substr($file['path'], $versionsBegin + 2);
-					$relpath = substr($file['path'], $relPathStart, $versionsBegin - $relPathStart);
+					$version = substr($filePath, $versionsBegin + 2);
+					$relpath = substr($filePath, $relPathStart, $versionsBegin - $relPathStart);
 					$key = $version . '#' . $relpath;
 					$versions[$key] = array('path' => $relpath, 'timestamp' => $version);
 				}
@@ -491,10 +555,36 @@ class Storage {
 	 * get list of files we want to expire
 	 * @param array $versions list of versions
 	 * @param integer $time
+	 * @param bool $quotaExceeded is versions storage limit reached
 	 * @return array containing the list of to deleted versions and the size of them
 	 */
-	protected static function getExpireList($time, $versions) {
+	protected static function getExpireList($time, $versions, $quotaExceeded = false) {
+		$expiration = self::getExpiration();
 
+		if ($expiration->shouldAutoExpire()) {
+			list($toDelete, $size) = self::getAutoExpireList($time, $versions);
+		} else {
+			$size = 0;
+			$toDelete = [];  // versions we want to delete
+		}
+
+		foreach ($versions as $key => $version) {
+			if ($expiration->isExpired($version['version'], $quotaExceeded) && !isset($toDelete[$key])) {
+				$size += $version['size'];
+				$toDelete[$key] = $version['path'] . '.v' . $version['version'];
+			}
+		}
+
+		return [$toDelete, $size];
+	}
+
+	/**
+	 * get list of files we want to expire
+	 * @param array $versions list of versions
+	 * @param integer $time
+	 * @return array containing the list of to deleted versions and the size of them
+	 */
+	protected static function getAutoExpireList($time, $versions) {
 		$size = 0;
 		$toDelete = array();  // versions we want to delete
 
@@ -520,7 +610,7 @@ class Storage {
 						//distance between two version too small, mark to delete
 						$toDelete[$key] = $version['path'] . '.v' . $version['version'];
 						$size += $version['size'];
-						\OCP\Util::writeLog('files_versions', 'Mark to expire '. $version['path'] .' next version should be ' . $nextVersion . " or smaller. (prevTimestamp: " . $prevTimestamp . "; step: " . $step, \OCP\Util::DEBUG);
+						\OCP\Util::writeLog('files_versions', 'Mark to expire '. $version['path'] .' next version should be ' . $nextVersion . " or smaller. (prevTimestamp: " . $prevTimestamp . "; step: " . $step, \OCP\Util::INFO);
 					} else {
 						$nextVersion = $version['version'] - $step;
 						$prevTimestamp = $version['version'];
@@ -541,7 +631,6 @@ class Storage {
 		}
 
 		return array($toDelete, $size);
-
 	}
 
 	/**
@@ -551,8 +640,12 @@ class Storage {
 	 * @param string $fileName file/folder for which to schedule expiration
 	 */
 	private static function scheduleExpire($uid, $fileName) {
-		$command = new Expire($uid, $fileName);
-		\OC::$server->getCommandBus()->push($command);
+		// let the admin disable auto expire
+		$expiration = self::getExpiration();
+		if ($expiration->isEnabled()) {
+			$command = new Expire($uid, $fileName);
+			\OC::$server->getCommandBus()->push($command);
+		}
 	}
 
 	/**
@@ -563,7 +656,9 @@ class Storage {
 	 */
 	public static function expire($filename) {
 		$config = \OC::$server->getConfig();
-		if($config->getSystemValue('files_versions', Storage::DEFAULTENABLED)=='true') {
+		$expiration = self::getExpiration();
+		
+		if($config->getSystemValue('files_versions', Storage::DEFAULTENABLED)=='true' && $expiration->isEnabled()) {
 			list($uid, $filename) = self::getUidAndFilename($filename);
 			if (empty($filename)) {
 				// file maybe renamed or deleted
@@ -589,23 +684,27 @@ class Storage {
 
 			// calculate available space for version history
 			// subtract size of files and current versions size from quota
-			if ($softQuota) {
-				$files_view = new \OC\Files\View('/'.$uid.'/files');
-				$rootInfo = $files_view->getFileInfo('/', false);
-				$free = $quota-$rootInfo['size']; // remaining free space for user
-				if ( $free > 0 ) {
-					$availableSpace = ($free * self::DEFAULTMAXSIZE / 100) - $versionsSize; // how much space can be used for versions
+			if ($quota >= 0) {
+				if ($softQuota) {
+					$files_view = new \OC\Files\View('/' . $uid . '/files');
+					$rootInfo = $files_view->getFileInfo('/', false);
+					$free = $quota - $rootInfo['size']; // remaining free space for user
+					if ($free > 0) {
+						$availableSpace = ($free * self::DEFAULTMAXSIZE / 100) - $versionsSize; // how much space can be used for versions
+					} else {
+						$availableSpace = $free - $versionsSize;
+					}
 				} else {
-					$availableSpace = $free - $versionsSize;
+					$availableSpace = $quota;
 				}
 			} else {
-				$availableSpace = $quota;
+				$availableSpace = PHP_INT_MAX;
 			}
 
 			$allVersions = Storage::getVersions($uid, $filename);
 
 			$time = time();
-			list($toDelete, $sizeOfDeletedVersions) = self::getExpireList($time, $allVersions);
+			list($toDelete, $sizeOfDeletedVersions) = self::getExpireList($time, $allVersions, $availableSpace <= 0);
 
 			$availableSpace = $availableSpace + $sizeOfDeletedVersions;
 			$versionsSize = $versionsSize - $sizeOfDeletedVersions;
@@ -616,7 +715,7 @@ class Storage {
 				$allVersions = $result['all'];
 
 				foreach ($result['by_file'] as $versions) {
-					list($toDeleteNew, $size) = self::getExpireList($time, $versions);
+					list($toDeleteNew, $size) = self::getExpireList($time, $versions, $availableSpace <= 0);
 					$toDelete = array_merge($toDelete, $toDeleteNew);
 					$sizeOfDeletedVersions += $size;
 				}
@@ -629,7 +728,7 @@ class Storage {
 				self::deleteVersion($versionsFileview, $path);
 				\OC_Hook::emit('\OCP\Versions', 'delete', array('path' => $path));
 				unset($allVersions[$key]); // update array with the versions we keep
-				\OCP\Util::writeLog('files_versions', "Expire: " . $path, \OCP\Util::DEBUG);
+				\OCP\Util::writeLog('files_versions', "Expire: " . $path, \OCP\Util::INFO);
 			}
 
 			// Check if enough space is available after versions are rearranged.
@@ -645,7 +744,7 @@ class Storage {
 				\OC_Hook::emit('\OCP\Versions', 'preDelete', array('path' => $version['path'].'.v'.$version['version']));
 				self::deleteVersion($versionsFileview, $version['path'] . '.v' . $version['version']);
 				\OC_Hook::emit('\OCP\Versions', 'delete', array('path' => $version['path'].'.v'.$version['version']));
-				\OCP\Util::writeLog('files_versions', 'running out of space! Delete oldest version: ' . $version['path'].'.v'.$version['version'] , \OCP\Util::DEBUG);
+				\OCP\Util::writeLog('files_versions', 'running out of space! Delete oldest version: ' . $version['path'].'.v'.$version['version'] , \OCP\Util::INFO);
 				$versionsSize -= $version['size'];
 				$availableSpace += $version['size'];
 				next($allVersions);
@@ -678,4 +777,15 @@ class Storage {
 		}
 	}
 
+	/**
+	 * Static workaround
+	 * @return Expiration
+	 */
+	protected static function getExpiration(){
+		if (is_null(self::$application)) {
+			self::$application = new Application();
+		}
+		return self::$application->getContainer()->query('Expiration');
+	}
+
 }
