@@ -1,26 +1,72 @@
 <?php
-
+/**
+ * @author Arthur Schiwon <blizzz@owncloud.com>
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Bernhard Posselt <dev@bernhard-posselt.com>
+ * @author Bernhard Reiter <ockham@raz.or.at>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Christopher Schäpers <kondou@ts.unde.re>
+ * @author Individual IT Services <info@individual-it.net>
+ * @author Joas Schilling <nickvergessen@owncloud.com>
+ * @author Jörn Friedrich Dreyer <jfd@butonic.de>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
+ * @author Roeland Jago Douma <roeland@famdouma.nl>
+ * @author Sander <brantje@gmail.com>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Thomas Tanghus <thomas@tanghus.net>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
+ */
 namespace OC;
 
 use bantu\IniGetWrapper\IniGetWrapper;
 use OC\AppFramework\Http\Request;
 use OC\AppFramework\Db\Db;
 use OC\AppFramework\Utility\SimpleContainer;
-use OC\Cache\UserCache;
-use OC\Diagnostics\NullQueryLogger;
+use OC\AppFramework\Utility\TimeFactory;
+use OC\Command\AsyncBus;
 use OC\Diagnostics\EventLogger;
+use OC\Diagnostics\NullEventLogger;
+use OC\Diagnostics\NullQueryLogger;
 use OC\Diagnostics\QueryLogger;
-use OC\Files\Config\StorageManager;
-use OC\Security\CertificateManager;
+use OC\Files\Node\HookConnector;
 use OC\Files\Node\Root;
 use OC\Files\View;
+use OC\Http\Client\ClientService;
+use OC\Lock\DBLockingProvider;
+use OC\Lock\MemcacheLockingProvider;
+use OC\Lock\NoopLockingProvider;
+use OC\Mail\Mailer;
+use OC\Memcache\ArrayCache;
+use OC\Notification\Manager;
+use OC\Security\CertificateManager;
 use OC\Security\Crypto;
 use OC\Security\Hasher;
 use OC\Security\SecureRandom;
-use OC\Diagnostics\NullEventLogger;
-use OCP\IServerContainer;
-use OCP\ISession;
+use OC\Security\TrustedDomainHelper;
+use OC\Session\CryptoWrapper;
 use OC\Tagging\TagMapper;
+use OCP\IServerContainer;
+use Symfony\Component\EventDispatcher\EventDispatcher;
+use Symfony\Component\EventDispatcher\EventDispatcherInterface;
 
 /**
  * Class Server
@@ -36,55 +82,59 @@ class Server extends SimpleContainer implements IServerContainer {
 	/**
 	 * @param string $webRoot
 	 */
-	function __construct($webRoot) {
+	public function __construct($webRoot) {
 		parent::__construct();
 		$this->webRoot = $webRoot;
 
 		$this->registerService('ContactsManager', function ($c) {
 			return new ContactsManager();
 		});
-		$this->registerService('Request', function (Server $c) {
-			if (isset($c['urlParams'])) {
-				$urlParams = $c['urlParams'];
-			} else {
-				$urlParams = array();
-			}
 
-			if ($c->getSession()->exists('requesttoken')) {
-				$requestToken = $c->getSession()->get('requesttoken');
-			} else {
-				$requestToken = false;
-			}
+		$this->registerService('PreviewManager', function (Server $c) {
+			return new PreviewManager($c->getConfig());
+		});
 
-			if (defined('PHPUNIT_RUN') && PHPUNIT_RUN
-				&& in_array('fakeinput', stream_get_wrappers())
-			) {
-				$stream = 'fakeinput://data';
-			} else {
-				$stream = 'php://input';
-			}
-
-			return new Request(
-				array(
-					'get' => $_GET,
-					'post' => $_POST,
-					'files' => $_FILES,
-					'server' => $_SERVER,
-					'env' => $_ENV,
-					'cookies' => $_COOKIE,
-					'method' => (isset($_SERVER) && isset($_SERVER['REQUEST_METHOD']))
-						? $_SERVER['REQUEST_METHOD']
-						: null,
-					'urlParams' => $urlParams,
-					'requesttoken' => $requestToken,
-				), $stream
+		$this->registerService('EncryptionManager', function (Server $c) {
+			$view = new View();
+			$util = new Encryption\Util(
+				$view,
+				$c->getUserManager(),
+				$c->getGroupManager(),
+				$c->getConfig()
+			);
+			return new Encryption\Manager(
+				$c->getConfig(),
+				$c->getLogger(),
+				$c->getL10N('core'),
+				new View(),
+				$util,
+				new ArrayCache()
 			);
 		});
-		$this->registerService('PreviewManager', function ($c) {
-			return new PreviewManager();
+
+		$this->registerService('EncryptionFileHelper', function (Server $c) {
+			$util = new Encryption\Util(
+				new View(),
+				$c->getUserManager(),
+				$c->getGroupManager(),
+				$c->getConfig()
+			);
+			return new Encryption\File($util);
+		});
+
+		$this->registerService('EncryptionKeyStorage', function (Server $c) {
+			$view = new View();
+			$util = new Encryption\Util(
+				$view,
+				$c->getUserManager(),
+				$c->getGroupManager(),
+				$c->getConfig()
+			);
+
+			return new Encryption\Keys\Storage($view, $util);
 		});
 		$this->registerService('TagMapper', function(Server $c) {
-			return new TagMapper($c->getDb());
+			return new TagMapper($c->getDatabaseConnection());
 		});
 		$this->registerService('TagManager', function (Server $c) {
 			$tagMapper = $c->query('TagMapper');
@@ -98,7 +148,10 @@ class Server extends SimpleContainer implements IServerContainer {
 			$user = $userManager->get($user);
 			$manager = \OC\Files\Filesystem::getMountManager();
 			$view = new View();
-			return new Root($manager, $view, $user);
+			$root = new Root($manager, $view, $user);
+			$connector = new HookConnector($root, $view);
+			$connector->viewToNode();
+			return $root;
 		});
 		$this->registerService('UserManager', function (Server $c) {
 			$config = $c->getConfig();
@@ -130,7 +183,10 @@ class Server extends SimpleContainer implements IServerContainer {
 		});
 		$this->registerService('UserSession', function (Server $c) {
 			$manager = $c->getUserManager();
-			$userSession = new \OC\User\Session($manager, new \OC\Session\Memory(''));
+
+			$session = new \OC\Session\Memory('');
+
+			$userSession = new \OC\User\Session($manager, $session);
 			$userSession->listen('\OC\User', 'preCreateUser', function ($uid, $password) {
 				\OC_Hook::emit('OC_User', 'pre_createUser', array('run' => true, 'uid' => $uid, 'password' => $password));
 			});
@@ -185,20 +241,47 @@ class Server extends SimpleContainer implements IServerContainer {
 		});
 		$this->registerService('URLGenerator', function (Server $c) {
 			$config = $c->getConfig();
-			return new \OC\URLGenerator($config);
+			$cacheFactory = $c->getMemCacheFactory();
+			return new \OC\URLGenerator(
+				$config,
+				$cacheFactory
+			);
 		});
 		$this->registerService('AppHelper', function ($c) {
 			return new \OC\AppHelper();
 		});
 		$this->registerService('UserCache', function ($c) {
-			return new UserCache();
+			return new Cache\File();
 		});
-		$this->registerService('MemCacheFactory', function ($c) {
-			$instanceId = \OC_Util::getInstanceId();
-			return new \OC\Memcache\Factory($instanceId);
+		$this->registerService('MemCacheFactory', function (Server $c) {
+			$config = $c->getConfig();
+
+			if($config->getSystemValue('installed', false) && !(defined('PHPUNIT_RUN') && PHPUNIT_RUN)) {
+				$v = \OC_App::getAppVersions();
+				$v['core'] = md5(file_get_contents(\OC::$SERVERROOT . '/version.php'));
+				$version = implode(',', $v);
+				$instanceId = \OC_Util::getInstanceId();
+				$path = \OC::$SERVERROOT;
+				$prefix = md5($instanceId.'-'.$version.'-'.$path);
+				return new \OC\Memcache\Factory($prefix, $c->getLogger(),
+					$config->getSystemValue('memcache.local', null),
+					$config->getSystemValue('memcache.distributed', null),
+					$config->getSystemValue('memcache.locking', null)
+				);
+			}
+
+			return new \OC\Memcache\Factory('', $c->getLogger(),
+				'\\OC\\Memcache\\ArrayCache',
+				'\\OC\\Memcache\\ArrayCache',
+				'\\OC\\Memcache\\ArrayCache'
+			);
 		});
-		$this->registerService('ActivityManager', function ($c) {
-			return new ActivityManager();
+		$this->registerService('ActivityManager', function (Server $c) {
+			return new ActivityManager(
+				$c->getRequest(),
+				$c->getUserSession(),
+				$c->getConfig()
+			);
 		});
 		$this->registerService('AvatarManager', function ($c) {
 			return new AvatarManager();
@@ -216,10 +299,11 @@ class Server extends SimpleContainer implements IServerContainer {
 		});
 		$this->registerService('Router', function (Server $c) {
 			$cacheFactory = $c->getMemCacheFactory();
+			$logger = $c->getLogger();
 			if ($cacheFactory->isAvailable()) {
-				$router = new \OC\Route\CachingRouter($cacheFactory->create('route'));
+				$router = new \OC\Route\CachingRouter($cacheFactory->create('route'), $logger);
 			} else {
-				$router = new \OC\Route\Router();
+				$router = new \OC\Route\Router($logger);
 			}
 			return $router;
 		});
@@ -252,32 +336,46 @@ class Server extends SimpleContainer implements IServerContainer {
 		});
 		$this->registerService('HTTPHelper', function (Server $c) {
 			$config = $c->getConfig();
-			$user = $c->getUserSession()->getUser();
-			$uid = $user ? $user->getUID() : null;
-			return new HTTPHelper($config, new \OC\Security\CertificateManager($uid, new \OC\Files\View()));
+			return new HTTPHelper(
+				$config,
+				$c->getHTTPClientService()
+			);
+		});
+		$this->registerService('HttpClientService', function (Server $c) {
+			$user = \OC_User::getUser();
+			$uid = $user ? $user : null;
+			return new ClientService(
+				$c->getConfig(),
+				new \OC\Security\CertificateManager($uid, new View(), $c->getConfig())
+			);
 		});
 		$this->registerService('EventLogger', function (Server $c) {
-			if (defined('DEBUG') and DEBUG) {
+			if ($c->getSystemConfig()->getValue('debug', false)) {
 				return new EventLogger();
 			} else {
 				return new NullEventLogger();
 			}
 		});
-		$this->registerService('QueryLogger', function ($c) {
-			if (defined('DEBUG') and DEBUG) {
+		$this->registerService('QueryLogger', function (Server $c) {
+			if ($c->getSystemConfig()->getValue('debug', false)) {
 				return new QueryLogger();
 			} else {
 				return new NullQueryLogger();
 			}
 		});
 		$this->registerService('TempManager', function (Server $c) {
-			return new TempManager(get_temp_dir(), $c->getLogger());
+			return new TempManager(
+				$c->getLogger(),
+				$c->getConfig()
+			);
 		});
 		$this->registerService('AppManager', function(Server $c) {
-			$userSession = $c->getUserSession();
-			$appConfig = $c->getAppConfig();
-			$groupManager = $c->getGroupManager();
-			return new \OC\App\AppManager($userSession, $appConfig, $groupManager);
+			return new \OC\App\AppManager(
+				$c->getUserSession(),
+				$c->getAppConfig(),
+				$c->getGroupManager(),
+				$c->getMemCacheFactory()
+			);
 		});
 		$this->registerService('DateTimeZone', function(Server $c) {
 			return new DateTimeZone(
@@ -300,23 +398,170 @@ class Server extends SimpleContainer implements IServerContainer {
 		$this->registerService('IniWrapper', function ($c) {
 			return new IniGetWrapper();
 		});
+		$this->registerService('AsyncCommandBus', function (Server $c) {
+			$jobList = $c->getJobList();
+			return new AsyncBus($jobList);
+		});
+		$this->registerService('TrustedDomainHelper', function ($c) {
+			return new TrustedDomainHelper($this->getConfig());
+		});
+		$this->registerService('Request', function ($c) {
+			if (isset($this['urlParams'])) {
+				$urlParams = $this['urlParams'];
+			} else {
+				$urlParams = [];
+			}
+
+			if ($this->getSession()->exists('requesttoken')) {
+				$requestToken = $this->getSession()->get('requesttoken');
+			} else {
+				$requestToken = false;
+			}
+
+			if (defined('PHPUNIT_RUN') && PHPUNIT_RUN
+				&& in_array('fakeinput', stream_get_wrappers())
+			) {
+				$stream = 'fakeinput://data';
+			} else {
+				$stream = 'php://input';
+			}
+
+			return new Request(
+				[
+					'get' => $_GET,
+					'post' => $_POST,
+					'files' => $_FILES,
+					'server' => $_SERVER,
+					'env' => $_ENV,
+					'cookies' => $_COOKIE,
+					'method' => (isset($_SERVER) && isset($_SERVER['REQUEST_METHOD']))
+						? $_SERVER['REQUEST_METHOD']
+						: null,
+					'urlParams' => $urlParams,
+					'requesttoken' => $requestToken,
+				],
+				$this->getSecureRandom(),
+				$this->getConfig(),
+				$stream
+			);
+		});
+		$this->registerService('Mailer', function(Server $c) {
+			return new Mailer(
+				$c->getConfig(),
+				$c->getLogger(),
+				new \OC_Defaults()
+			);
+		});
+		$this->registerService('OcsClient', function(Server $c) {
+			return new OCSClient(
+				$this->getHTTPClientService(),
+				$this->getConfig(),
+				$this->getLogger()
+			);
+		});
+		$this->registerService('LockingProvider', function (Server $c) {
+			if ($c->getConfig()->getSystemValue('filelocking.enabled', true) or (defined('PHPUNIT_RUN') && PHPUNIT_RUN)) {
+				/** @var \OC\Memcache\Factory $memcacheFactory */
+				$memcacheFactory = $c->getMemCacheFactory();
+				$memcache = $memcacheFactory->createLocking('lock');
+				if (!($memcache instanceof \OC\Memcache\NullCache)) {
+					return new MemcacheLockingProvider($memcache);
+				}
+				return new DBLockingProvider($c->getDatabaseConnection(), $c->getLogger(), new TimeFactory());
+			}
+			return new NoopLockingProvider();
+		});
+		$this->registerService('MountManager', function () {
+			return new \OC\Files\Mount\Manager();
+		});
+		$this->registerService('MimeTypeDetector', function(Server $c) {
+			return new \OC\Files\Type\Detection(
+				$c->getURLGenerator(),
+				\OC::$SERVERROOT . '/config/',
+				\OC::$SERVERROOT . '/resources/config/'
+				);
+		});
+		$this->registerService('MimeTypeLoader', function(Server $c) {
+			return new \OC\Files\Type\Loader(
+				$c->getDatabaseConnection()
+			);
+		});
+		$this->registerService('NotificationManager', function() {
+			return new Manager();
+		});
+		$this->registerService('CapabilitiesManager', function (Server $c) {
+			$manager = new \OC\CapabilitiesManager();
+			$manager->registerCapability(function() use ($c) {
+				return new \OC\OCS\CoreCapabilities($c->getConfig());
+			});
+			return $manager;
+		});
+		$this->registerService('EventDispatcher', function() {
+			return new EventDispatcher();
+		});
+		$this->registerService('CryptoWrapper', function (Server $c) {
+			// FIXME: Instantiiated here due to cyclic dependency
+			$request = new Request(
+				[
+					'get' => $_GET,
+					'post' => $_POST,
+					'files' => $_FILES,
+					'server' => $_SERVER,
+					'env' => $_ENV,
+					'cookies' => $_COOKIE,
+					'method' => (isset($_SERVER) && isset($_SERVER['REQUEST_METHOD']))
+						? $_SERVER['REQUEST_METHOD']
+						: null,
+				],
+				new SecureRandom(),
+				$c->getConfig()
+			);
+
+			return new CryptoWrapper(
+				$c->getConfig(),
+				$c->getCrypto(),
+				$c->getSecureRandom(),
+				$request
+			);
+		});
 	}
 
 	/**
 	 * @return \OCP\Contacts\IManager
 	 */
-	function getContactsManager() {
+	public function getContactsManager() {
 		return $this->query('ContactsManager');
 	}
 
+	/**
+	 * @return \OC\Encryption\Manager
+	 */
+	public function getEncryptionManager() {
+		return $this->query('EncryptionManager');
+	}
+
+	/**
+	 * @return \OC\Encryption\File
+	 */
+	public function getEncryptionFilesHelper() {
+		return $this->query('EncryptionFileHelper');
+	}
+
+	/**
+	 * @return \OCP\Encryption\Keys\IStorage
+	 */
+	public function getEncryptionKeyStorage() {
+		return $this->query('EncryptionKeyStorage');
+	}
+
 	/**
 	 * The current request object holding all information about the request
 	 * currently being processed is returned from this method.
 	 * In case the current execution was not initiated by a web request null is returned
 	 *
-	 * @return \OCP\IRequest|null
+	 * @return \OCP\IRequest
 	 */
-	function getRequest() {
+	public function getRequest() {
 		return $this->query('Request');
 	}
 
@@ -325,7 +570,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\IPreview
 	 */
-	function getPreviewManager() {
+	public function getPreviewManager() {
 		return $this->query('PreviewManager');
 	}
 
@@ -335,7 +580,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 * @see \OCP\ITagManager::load()
 	 * @return \OCP\ITagManager
 	 */
-	function getTagManager() {
+	public function getTagManager() {
 		return $this->query('TagManager');
 	}
 
@@ -344,16 +589,16 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\IAvatarManager
 	 */
-	function getAvatarManager() {
+	public function getAvatarManager() {
 		return $this->query('AvatarManager');
 	}
 
 	/**
 	 * Returns the root folder of ownCloud's data directory
 	 *
-	 * @return \OCP\Files\Folder
+	 * @return \OCP\Files\IRootFolder
 	 */
-	function getRootFolder() {
+	public function getRootFolder() {
 		return $this->query('RootFolder');
 	}
 
@@ -363,48 +608,16 @@ class Server extends SimpleContainer implements IServerContainer {
 	 * @param string $userId user ID
 	 * @return \OCP\Files\Folder
 	 */
-	function getUserFolder($userId = null) {
+	public function getUserFolder($userId = null) {
 		if ($userId === null) {
 			$user = $this->getUserSession()->getUser();
 			if (!$user) {
 				return null;
 			}
 			$userId = $user->getUID();
-		} else {
-			$user = $this->getUserManager()->get($userId);
 		}
-		\OC\Files\Filesystem::initMountPoints($userId);
-		$dir = '/' . $userId;
 		$root = $this->getRootFolder();
-		$folder = null;
-
-		if (!$root->nodeExists($dir)) {
-			$folder = $root->newFolder($dir);
-		} else {
-			$folder = $root->get($dir);
-		}
-
-		$dir = '/files';
-		if (!$folder->nodeExists($dir)) {
-			$folder = $folder->newFolder($dir);
-
-			if (\OCP\App::isEnabled('files_encryption')) {
-				// disable encryption proxy to prevent recursive calls
-				$proxyStatus = \OC_FileProxy::$enabled;
-				\OC_FileProxy::$enabled = false;
-			}
-
-			\OC_Util::copySkeleton($user, $folder);
-
-			if (\OCP\App::isEnabled('files_encryption')) {
-				// re-enable proxy - our work is done
-				\OC_FileProxy::$enabled = $proxyStatus;
-			}
-		} else {
-			$folder = $folder->get($dir);
-		}
-
-		return $folder;
+		return $root->getUserFolder($userId);
 	}
 
 	/**
@@ -412,7 +625,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Files\Folder
 	 */
-	function getAppFolder() {
+	public function getAppFolder() {
 		$dir = '/' . \OC_App::getCurrentApp();
 		$root = $this->getRootFolder();
 		$folder = null;
@@ -427,49 +640,49 @@ class Server extends SimpleContainer implements IServerContainer {
 	/**
 	 * @return \OC\User\Manager
 	 */
-	function getUserManager() {
+	public function getUserManager() {
 		return $this->query('UserManager');
 	}
 
 	/**
 	 * @return \OC\Group\Manager
 	 */
-	function getGroupManager() {
+	public function getGroupManager() {
 		return $this->query('GroupManager');
 	}
 
 	/**
 	 * @return \OC\User\Session
 	 */
-	function getUserSession() {
+	public function getUserSession() {
 		return $this->query('UserSession');
 	}
 
 	/**
 	 * @return \OCP\ISession
 	 */
-	function getSession() {
+	public function getSession() {
 		return $this->query('UserSession')->getSession();
 	}
 
 	/**
 	 * @param \OCP\ISession $session
 	 */
-	function setSession(\OCP\ISession $session) {
+	public function setSession(\OCP\ISession $session) {
 		return $this->query('UserSession')->setSession($session);
 	}
 
 	/**
 	 * @return \OC\NavigationManager
 	 */
-	function getNavigationManager() {
+	public function getNavigationManager() {
 		return $this->query('NavigationManager');
 	}
 
 	/**
 	 * @return \OCP\IConfig
 	 */
-	function getConfig() {
+	public function getConfig() {
 		return $this->query('AllConfig');
 	}
 
@@ -478,7 +691,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OC\SystemConfig
 	 */
-	function getSystemConfig() {
+	public function getSystemConfig() {
 		return $this->query('SystemConfig');
 	}
 
@@ -487,10 +700,17 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\IAppConfig
 	 */
-	function getAppConfig() {
+	public function getAppConfig() {
 		return $this->query('AppConfig');
 	}
 
+	/**
+	 * @return \OCP\L10N\IFactory
+	 */
+	public function getL10NFactory() {
+		return $this->query('L10NFactory');
+	}
+
 	/**
 	 * get an L10N instance
 	 *
@@ -498,30 +718,32 @@ class Server extends SimpleContainer implements IServerContainer {
 	 * @param string $lang
 	 * @return \OC_L10N
 	 */
-	function getL10N($app, $lang = null) {
-		return $this->query('L10NFactory')->get($app, $lang);
+	public function getL10N($app, $lang = null) {
+		return $this->getL10NFactory()->get($app, $lang);
 	}
 
 	/**
 	 * @return \OCP\IURLGenerator
 	 */
-	function getURLGenerator() {
+	public function getURLGenerator() {
 		return $this->query('URLGenerator');
 	}
 
 	/**
 	 * @return \OCP\IHelper
 	 */
-	function getHelper() {
+	public function getHelper() {
 		return $this->query('AppHelper');
 	}
 
 	/**
-	 * Returns an ICache instance
+	 * Returns an ICache instance. Since 8.1.0 it returns a fake cache. Use
+	 * getMemCacheFactory() instead.
 	 *
 	 * @return \OCP\ICache
+	 * @deprecated 8.1.0 use getMemCacheFactory to obtain a proper cache
 	 */
-	function getCache() {
+	public function getCache() {
 		return $this->query('UserCache');
 	}
 
@@ -530,7 +752,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\ICacheFactory
 	 */
-	function getMemCacheFactory() {
+	public function getMemCacheFactory() {
 		return $this->query('MemCacheFactory');
 	}
 
@@ -539,7 +761,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\IDBConnection
 	 */
-	function getDatabaseConnection() {
+	public function getDatabaseConnection() {
 		return $this->query('DatabaseConnection');
 	}
 
@@ -548,7 +770,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Activity\IManager
 	 */
-	function getActivityManager() {
+	public function getActivityManager() {
 		return $this->query('ActivityManager');
 	}
 
@@ -557,7 +779,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\BackgroundJob\IJobList
 	 */
-	function getJobList() {
+	public function getJobList() {
 		return $this->query('JobList');
 	}
 
@@ -566,7 +788,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\ILogger
 	 */
-	function getLogger() {
+	public function getLogger() {
 		return $this->query('Logger');
 	}
 
@@ -575,7 +797,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Route\IRouter
 	 */
-	function getRouter() {
+	public function getRouter() {
 		return $this->query('Router');
 	}
 
@@ -584,7 +806,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\ISearch
 	 */
-	function getSearch() {
+	public function getSearch() {
 		return $this->query('Search');
 	}
 
@@ -593,7 +815,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Security\ISecureRandom
 	 */
-	function getSecureRandom() {
+	public function getSecureRandom() {
 		return $this->query('SecureRandom');
 	}
 
@@ -602,7 +824,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Security\ICrypto
 	 */
-	function getCrypto() {
+	public function getCrypto() {
 		return $this->query('Crypto');
 	}
 
@@ -611,44 +833,53 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Security\IHasher
 	 */
-	function getHasher() {
+	public function getHasher() {
 		return $this->query('Hasher');
 	}
 
 	/**
 	 * Returns an instance of the db facade
-	 *
+	 * @deprecated use getDatabaseConnection, will be removed in ownCloud 10
 	 * @return \OCP\IDb
 	 */
-	function getDb() {
+	public function getDb() {
 		return $this->query('Db');
 	}
 
 	/**
 	 * Returns an instance of the HTTP helper class
-	 *
+	 * @deprecated Use getHTTPClientService()
 	 * @return \OC\HTTPHelper
 	 */
-	function getHTTPHelper() {
+	public function getHTTPHelper() {
 		return $this->query('HTTPHelper');
 	}
 
 	/**
 	 * Get the certificate manager for the user
 	 *
-	 * @param string $uid (optional) if not specified the current loggedin user is used
+	 * @param string $userId (optional) if not specified the current loggedin user is used
 	 * @return \OCP\ICertificateManager | null if $uid is null and no user is logged in
 	 */
-	function getCertificateManager($uid = null) {
-		if (is_null($uid)) {
+	public function getCertificateManager($userId = null) {
+		if (is_null($userId)) {
 			$userSession = $this->getUserSession();
 			$user = $userSession->getUser();
 			if (is_null($user)) {
 				return null;
 			}
-			$uid = $user->getUID();
+			$userId = $user->getUID();
 		}
-		return new CertificateManager($uid, new \OC\Files\View());
+		return new CertificateManager($userId, new View(), $this->getConfig());
+	}
+
+	/**
+	 * Returns an instance of the HTTP client service
+	 *
+	 * @return \OCP\Http\Client\IClientService
+	 */
+	public function getHTTPClientService() {
+		return $this->query('HttpClientService');
 	}
 
 	/**
@@ -656,7 +887,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\IEventSource
 	 */
-	function createEventSource() {
+	public function createEventSource() {
 		return new \OC_EventSource();
 	}
 
@@ -667,7 +898,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Diagnostics\IEventLogger
 	 */
-	function getEventLogger() {
+	public function getEventLogger() {
 		return $this->query('EventLogger');
 	}
 
@@ -678,7 +909,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\Diagnostics\IQueryLogger
 	 */
-	function getQueryLogger() {
+	public function getQueryLogger() {
 		return $this->query('QueryLogger');
 	}
 
@@ -687,7 +918,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\ITempManager
 	 */
-	function getTempManager() {
+	public function getTempManager() {
 		return $this->query('TempManager');
 	}
 
@@ -696,19 +927,35 @@ class Server extends SimpleContainer implements IServerContainer {
 	 *
 	 * @return \OCP\App\IAppManager
 	 */
-	function getAppManager() {
+	public function getAppManager() {
 		return $this->query('AppManager');
 	}
 
+	/**
+	 * Creates a new mailer
+	 *
+	 * @return \OCP\Mail\IMailer
+	 */
+	public function getMailer() {
+		return $this->query('Mailer');
+	}
+
 	/**
 	 * Get the webroot
 	 *
 	 * @return string
 	 */
-	function getWebRoot() {
+	public function getWebRoot() {
 		return $this->webRoot;
 	}
 
+	/**
+	 * @return \OC\OCSClient
+	 */
+	public function getOcsClient() {
+		return $this->query('OcsClient');
+	}
+
 	/**
 	 * @return \OCP\IDateTimeZone
 	 */
@@ -726,7 +973,7 @@ class Server extends SimpleContainer implements IServerContainer {
 	/**
 	 * @return \OCP\Files\Config\IMountProviderCollection
 	 */
-	function getMountProviderCollection(){
+	public function getMountProviderCollection(){
 		return $this->query('MountConfigManager');
 	}
 
@@ -738,4 +985,124 @@ class Server extends SimpleContainer implements IServerContainer {
 	public function getIniWrapper() {
 		return $this->query('IniWrapper');
 	}
+
+	/**
+	 * @return \OCP\Command\IBus
+	 */
+	public function getCommandBus(){
+		return $this->query('AsyncCommandBus');
+	}
+
+	/**
+	 * Get the trusted domain helper
+	 *
+	 * @return TrustedDomainHelper
+	 */
+	public function getTrustedDomainHelper() {
+		return $this->query('TrustedDomainHelper');
+	}
+
+	/**
+	 * Get the locking provider
+	 *
+	 * @return \OCP\Lock\ILockingProvider
+	 * @since 8.1.0
+	 */
+	public function getLockingProvider() {
+		return $this->query('LockingProvider');
+	}
+
+	/**
+	 * @return \OCP\Files\Mount\IMountManager
+	 **/
+	function getMountManager() {
+		return $this->query('MountManager');
+	}
+
+	/*
+	 * Get the MimeTypeDetector
+	 *
+	 * @return \OCP\Files\IMimeTypeDetector
+	 */
+	public function getMimeTypeDetector() {
+		return $this->query('MimeTypeDetector');
+	}
+
+	/**
+	 * Get the MimeTypeLoader
+	 *
+	 * @return \OCP\Files\IMimeTypeLoader
+	 */
+	public function getMimeTypeLoader() {
+		return $this->query('MimeTypeLoader');
+	}
+
+	/**
+	 * Get the manager of all the capabilities
+	 *
+	 * @return \OC\CapabilitiesManager
+	 */
+	public function getCapabilitiesManager() {
+		return $this->query('CapabilitiesManager');
+	}
+
+	/**
+	 * Get the EventDispatcher
+	 *
+	 * @return EventDispatcherInterface
+	 * @since 8.2.0
+	 */
+	public function getEventDispatcher() {
+		return $this->query('EventDispatcher');
+	}
+
+	/**
+	 * Get the Notification Manager
+	 *
+	 * @return \OC\Notification\IManager
+	 * @since 8.2.0
+	 */
+	public function getNotificationManager() {
+		return $this->query('NotificationManager');
+	}
+
+	/**
+	 * @return \OC\Session\CryptoWrapper
+	 */
+	public function getSessionCryptoWrapper() {
+		return $this->query('CryptoWrapper');
+	}
+
+	/**
+	 * Not a public API as of 8.2, wait for 9.0
+	 * @return \OCA\Files_External\Service\BackendService
+	 */
+	public function getStoragesBackendService() {
+		return \OC_Mount_Config::$app->getContainer()->query('OCA\\Files_External\\Service\\BackendService');
+	}
+
+	/**
+	 * Not a public API as of 8.2, wait for 9.0
+	 * @return \OCA\Files_External\Service\GlobalStoragesService
+	 */
+	public function getGlobalStoragesService() {
+		return \OC_Mount_Config::$app->getContainer()->query('OCA\\Files_External\\Service\\GlobalStoragesService');
+	}
+
+	/**
+	 * Not a public API as of 8.2, wait for 9.0
+	 * @return \OCA\Files_External\Service\UserGlobalStoragesService
+	 */
+	public function getUserGlobalStoragesService() {
+		return \OC_Mount_Config::$app->getContainer()->query('OCA\\Files_External\\Service\\UserGlobalStoragesService');
+	}
+
+	/**
+	 * Not a public API as of 8.2, wait for 9.0
+	 * @return \OCA\Files_External\Service\UserStoragesService
+	 */
+	public function getUserStoragesService() {
+		return \OC_Mount_Config::$app->getContainer()->query('OCA\\Files_External\\Service\\UserStoragesService');
+	}
+	
 }
