@@ -73,7 +73,7 @@ if ($_['getenvServerNotWorking']) {
 ?>
 	<li>
 		<?php p($l->t('php does not seem to be setup properly to query system environment variables. The test with getenv("PATH") only returns an empty response.')); ?><br>
-		<?php p($l->t('Please check the installation documentation for php configuration notes and the php configuration of your server, especially when using php-fpm.')); ?>
+		<?php print_unescaped($l->t('Please check the <a target="_blank" href="%s">installation documentation ↗</a> for php configuration notes and the php configuration of your server, especially when using php-fpm.', link_to_docs('admin-php-fpm'))); ?>
 	</li>
 <?php
 }
@@ -133,6 +133,15 @@ if (!$_['has_fileinfo']) {
 <?php
 }
 
+// locking configured optimally?
+if ($_['fileLockingType'] === 'none') {
+	?>
+	<li>
+		<?php print_unescaped($l->t('Transactional file locking is disabled, this might lead to issues with race conditions. Enable \'filelocking.enabled\' in config.php to avoid these problems. See the <a target="_blank" href="%s">documentation ↗</a> for more information.', link_to_docs('admin-transactional-locking'))); ?>
+	</li>
+	<?php
+}
+
 // is locale working ?
 if (!$_['isLocaleWorking']) {
 	?>
@@ -180,10 +189,15 @@ if ($_['cronErrors']) {
 <div id="postsetupchecks">
 	<div class="loading"></div>
 	<ul class="errors hidden"></ul>
+	<ul class="warnings hidden"></ul>
+	<ul class="info hidden"></ul>
 	<p class="hint hidden">
 		<?php print_unescaped($l->t('Please double check the <a target="_blank" href="%s">installation guides ↗</a>, and check for any errors or warnings in the <a href="#log-section">log</a>.', link_to_docs('admin-install'))); ?>
 	</p>
 </div>
+<div id="security-warning-state">
+	<span class="hidden icon-checkmark"><?php p($l->t('All checks passed.'));?></span>
+</div>
 </div>
 
 	<div class="section" id="shareAPI">
@@ -192,30 +206,30 @@ if ($_['cronErrors']) {
 			title="<?php p($l->t('Open documentation'));?>"
 			href="<?php p(link_to_docs('admin-sharing')); ?>"></a>
 		<p id="enable">
-			<input type="checkbox" name="shareapi_enabled" id="shareAPIEnabled"
+			<input type="checkbox" name="shareapi_enabled" id="shareAPIEnabled" class="checkbox"
 				   value="1" <?php if ($_['shareAPIEnabled'] === 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="shareAPIEnabled"><?php p($l->t('Allow apps to use the Share API'));?></label><br/>
 		</p>
 		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_allow_links" id="allowLinks"
+			<input type="checkbox" name="shareapi_allow_links" id="allowLinks" class="checkbox"
 				   value="1" <?php if ($_['allowLinks'] === 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="allowLinks"><?php p($l->t('Allow users to share via link'));?></label><br/>
 		</p>
 
 		<p id="publicLinkSettings" class="indent <?php if ($_['allowLinks'] !== 'yes' || $_['shareAPIEnabled'] === 'no') p('hidden'); ?>">
-			<input type="checkbox" name="shareapi_enforce_links_password" id="enforceLinkPassword"
+			<input type="checkbox" name="shareapi_enforce_links_password" id="enforceLinkPassword" class="checkbox"
 				   value="1" <?php if ($_['enforceLinkPassword']) print_unescaped('checked="checked"'); ?> />
 			<label for="enforceLinkPassword"><?php p($l->t('Enforce password protection'));?></label><br/>
 
-			<input type="checkbox" name="shareapi_allow_public_upload" id="allowPublicUpload"
+			<input type="checkbox" name="shareapi_allow_public_upload" id="allowPublicUpload" class="checkbox"
 				   value="1" <?php if ($_['allowPublicUpload'] == 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="allowPublicUpload"><?php p($l->t('Allow public uploads'));?></label><br/>
 
-			<input type="checkbox" name="shareapi_allow_public_notification" id="allowPublicMailNotification"
+			<input type="checkbox" name="shareapi_allow_public_notification" id="allowPublicMailNotification" class="checkbox"
 				   value="1" <?php if ($_['allowPublicMailNotification'] == 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="allowPublicMailNotification"><?php p($l->t('Allow users to send mail notification for shared files'));?></label><br/>
 
-			<input type="checkbox" name="shareapi_default_expire_date" id="shareapiDefaultExpireDate"
+			<input type="checkbox" name="shareapi_default_expire_date" id="shareapiDefaultExpireDate" class="checkbox"
 				   value="1" <?php if ($_['shareDefaultExpireDateSet'] === 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="shareapiDefaultExpireDate"><?php p($l->t('Set default expiration date'));?></label><br/>
 
@@ -225,27 +239,27 @@ if ($_['cronErrors']) {
 			<input type="text" name='shareapi_expire_after_n_days' id="shareapiExpireAfterNDays" placeholder="<?php p('7')?>"
 				   value='<?php p($_['shareExpireAfterNDays']) ?>' />
 			<?php p($l->t( 'days' )); ?>
-			<input type="checkbox" name="shareapi_enforce_expire_date" id="shareapiEnforceExpireDate"
+			<input type="checkbox" name="shareapi_enforce_expire_date" id="shareapiEnforceExpireDate" class="checkbox"
 				   value="1" <?php if ($_['shareEnforceExpireDate'] === 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="shareapiEnforceExpireDate"><?php p($l->t('Enforce expiration date'));?></label><br/>
 		</p>
 		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_allow_resharing" id="allowResharing"
+			<input type="checkbox" name="shareapi_allow_resharing" id="allowResharing" class="checkbox"
 				   value="1" <?php if ($_['allowResharing'] === 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="allowResharing"><?php p($l->t('Allow resharing'));?></label><br/>
 		</p>
 		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_only_share_with_group_members" id="onlyShareWithGroupMembers"
+			<input type="checkbox" name="shareapi_only_share_with_group_members" id="onlyShareWithGroupMembers" class="checkbox"
 				   value="1" <?php if ($_['onlyShareWithGroupMembers']) print_unescaped('checked="checked"'); ?> />
 			<label for="onlyShareWithGroupMembers"><?php p($l->t('Restrict users to only share with users in their groups'));?></label><br/>
 		</p>
 		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_allow_mail_notification" id="allowMailNotification"
+			<input type="checkbox" name="shareapi_allow_mail_notification" id="allowMailNotification" class="checkbox"
 				   value="1" <?php if ($_['allowMailNotification'] === 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="allowMailNotification"><?php p($l->t('Allow users to send mail notification for shared files to other users'));?></label><br/>
 		</p>
 		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_exclude_groups" id="shareapiExcludeGroups"
+			<input type="checkbox" name="shareapi_exclude_groups" id="shareapiExcludeGroups" class="checkbox"
 				   value="1" <?php if ($_['shareExcludeGroups']) print_unescaped('checked="checked"'); ?> />
 			<label for="shareapiExcludeGroups"><?php p($l->t('Exclude groups from sharing'));?></label><br/>
 		</p>
@@ -255,7 +269,7 @@ if ($_['cronErrors']) {
 			<em><?php p($l->t('These groups will still be able to receive shares, but not to initiate them.')); ?></em>
 		</p>
 		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_allow_share_dialog_user_enumeration" value="1" id="shareapi_allow_share_dialog_user_enumeration"
+			<input type="checkbox" name="shareapi_allow_share_dialog_user_enumeration" value="1" id="shareapi_allow_share_dialog_user_enumeration" class="checkbox"
 				<?php if ($_['allowShareDialogUserEnumeration'] === 'yes') print_unescaped('checked="checked"'); ?> />
 			<label for="shareapi_allow_share_dialog_user_enumeration"><?php p($l->t('Allow username autocompletion in share dialog. If this is disabled the full username needs to be entered.'));?></label><br />
 		</p>
@@ -333,17 +347,24 @@ if ($_['cronErrors']) {
 
 	<p id="enable">
 		<input type="checkbox"
-			   id="enableEncryption"
+			   id="enableEncryption" class="checkbox"
 			   value="1" <?php if ($_['encryptionEnabled']) print_unescaped('checked="checked" disabled="disabled"'); ?> />
 		<label
 			for="enableEncryption"><?php p($l->t('Enable server-side encryption')); ?> <span id="startmigration_msg" class="msg"></span> </label><br/>
 	</p>
 
 	<div id="EncryptionWarning" class="warning hidden">
-		<?php p($l->t('Encryption is a one way process. Once encryption is enabled, all files from that point forward will be encrypted on the server and it will not be possible to disable encryption at a later date. This is the final warning: Do you really want to enable encryption?')) ?>
-		<input type="button"
+		<p><?php p($l->t('Please read carefully before activating server-side encryption: ')); ?></p>
+		<ul>
+			<li><?php p($l->t('Once encryption is enabled, all files uploaded to the server from that point forward will be encrypted at rest on the server. It will only be possible to disable encryption at a later date if the active encryption module supports that function, and all pre-conditions (e.g. setting a recover key) are met.')); ?></li>
+			<li><?php p($l->t('Encryption alone does not guarantee security of the system. Please see ownCloud documentation for more information about how the encryption app works, and the supported use cases.')); ?></li>
+			<li><?php p($l->t('Be aware that encryption always increases the file size.')); ?></li>
+			<li><?php p($l->t('It is always good to create regular backups of your data, in case of encryption make sure to backup the encryption keys along with your data.')); ?></li>
+		</ul>
+
+		<p><?php p($l->t('This is the final warning: Do you really want to enable encryption?')) ?> <input type="button"
 			   id="reallyEnableEncryption"
-			   value="<?php p($l->t("Enable encryption")); ?>" />
+			   value="<?php p($l->t("Enable encryption")); ?>" /></p>
 	</div>
 
 	<div id="EncryptionSettingsArea" class="<?php if (!$_['encryptionEnabled']) p('hidden'); ?>">
@@ -440,7 +461,7 @@ if ($_['cronErrors']) {
 				<?php endforeach;?>
 			</select>
 
-			<input type="checkbox" name="mail_smtpauth" id="mail_smtpauth" value="1"
+			<input type="checkbox" name="mail_smtpauth" id="mail_smtpauth" class="checkbox" value="1"
 				   <?php if ($_['mail_smtpauth']) print_unescaped('checked="checked"'); ?> />
 			<label for="mail_smtpauth"><?php p($l->t( 'Authentication required' )); ?></label>
 		</p>
@@ -542,18 +563,10 @@ if ($_['cronErrors']) {
 		<li><a target="_blank" href="<?php p(link_to_docs('admin-security')); ?>"><?php p($l->t('Hardening and security guidance'));?> ↗</a></li>
 	</ul>
 </div>
-<div class="section" id="server-status">
-	<h2><?php p($l->t('Server Status'));?></h2>
-	<ul>
-		<li>
-			<?php if ($_['fileLockingEnabled']) {
-				p($l->t('Transactional File Locking is enabled.'));
-			} else {
-				p($l->t('Transactional File Locking is disabled.'));
-			} ?>
-		</li>
-	</ul>
-</div>
+
+<?php if (!empty($_['updaterAppPanel'])): ?>
+	<div id="updater"><?php print_unescaped($_['updaterAppPanel']); ?></div>
+<?php endif; ?>
 
 <div class="section">
 	<h2><?php p($l->t('Version'));?></h2>
@@ -561,10 +574,6 @@ if ($_['cronErrors']) {
 	<?php include('settings.development.notice.php'); ?>
 </div>
 
-<?php if (!empty($_['updaterAppPanel'])): ?>
-	<div id="updater"><?php print_unescaped($_['updaterAppPanel']); ?></div>
-<?php endif; ?>
-
 <div class="section credits-footer">
 	<p><?php print_unescaped($theme->getShortFooter()); ?></p>
 </div>
